/*
 *  (GLABELS) Label and Business Card Creation program for GNOME
 *
 *  wdgt_size.c:  size properties widget module
 *
 *  Copyright (C) 2001-2002  Jim Evins <evins@snaught.com>.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#include <config.h>

#include "wdgt-size.h"
#include "prefs.h"
#include "marshal.h"

#include "debug.h"

/*===========================================*/
/* Private types                             */
/*===========================================*/

enum {
	CHANGED,
	LAST_SIGNAL
};

typedef void (*glWdgtSizeSignal) (GObject * object, gpointer data);

/*===========================================*/
/* Private globals                           */
/*===========================================*/

static glHigVBoxClass *parent_class;

static gint wdgt_size_signals[LAST_SIGNAL] = { 0 };

/*===========================================*/
/* Local function prototypes                 */
/*===========================================*/

static void gl_wdgt_size_class_init    (glWdgtSizeClass *class);
static void gl_wdgt_size_instance_init (glWdgtSize      *size);
static void gl_wdgt_size_finalize      (GObject         *object);
static void gl_wdgt_size_construct     (glWdgtSize      *size);

static void aspect_toggle_cb (GtkToggleButton *togglebutton,
			      gpointer         user_data);

static void w_spin_cb        (GtkSpinButton    *spinbutton,
			      gpointer          user_data);
static void h_spin_cb        (GtkSpinButton    *spinbutton,
			      gpointer          user_data);

/***************************************************************************/
/* Boilerplate Object stuff.                                               */
/***************************************************************************/
guint
gl_wdgt_size_get_type (void)
{
	static guint wdgt_size_type = 0;

	if (!wdgt_size_type) {
		GTypeInfo wdgt_size_info = {
			sizeof (glWdgtSizeClass),
			NULL,
			NULL,
			(GClassInitFunc) gl_wdgt_size_class_init,
			NULL,
			NULL,
			sizeof (glWdgtSize),
			0,
			(GInstanceInitFunc) gl_wdgt_size_instance_init,
		};

		wdgt_size_type =
			g_type_register_static (gl_hig_vbox_get_type (),
						"glWdgtSize",
						&wdgt_size_info, 0);
	}

	return wdgt_size_type;
}

static void
gl_wdgt_size_class_init (glWdgtSizeClass *class)
{
	GObjectClass *object_class;

	object_class = (GObjectClass *) class;

	parent_class = g_type_class_peek_parent (class);

	object_class->finalize = gl_wdgt_size_finalize;

	wdgt_size_signals[CHANGED] =
	    g_signal_new ("changed",
			  G_OBJECT_CLASS_TYPE(object_class),
			  G_SIGNAL_RUN_LAST,
			  G_STRUCT_OFFSET (glWdgtSizeClass, changed),
			  NULL, NULL,
			  gl_marshal_VOID__VOID,
			  G_TYPE_NONE, 0);

}

static void
gl_wdgt_size_instance_init (glWdgtSize *size)
{
	size->aspect_ratio = 1.0;

	size->w_label = NULL;
	size->w_spin = NULL;
	size->h_label = NULL;
	size->h_spin = NULL;

	size->units_label = NULL;

	size->aspect_checkbox = NULL;
}

static void
gl_wdgt_size_finalize (GObject *object)
{
	glWdgtSize *size;
	glWdgtSizeClass *class;

	g_return_if_fail (object != NULL);
	g_return_if_fail (GL_IS_WDGT_SIZE (object));

	size = GL_WDGT_SIZE (object);

	G_OBJECT_CLASS (parent_class)->finalize (object);
}

/***************************************************************************/
/* New widget.                                                             */
/***************************************************************************/
GtkWidget *
gl_wdgt_size_new (void)
{
	glWdgtSize *size;

	size = g_object_new (gl_wdgt_size_get_type (), NULL);

	gl_wdgt_size_construct (size);

	return GTK_WIDGET (size);
}

/*-------------------------------------------------------------------------*/
/* PRIVATE.  Construct composite widget.                                   */
/*-------------------------------------------------------------------------*/
static void
gl_wdgt_size_construct (glWdgtSize *size)
{
	GtkWidget *wvbox, *whbox;
	GtkObject *w_adjust, *h_adjust;
	const gchar *units_string;
	gdouble units_per_point, climb_rate;
	gint digits;

	units_string = gl_prefs_get_units_string ();
	units_per_point = gl_prefs_get_units_per_point ();
	climb_rate = gl_prefs_get_units_step_size ();
	digits = gl_prefs_get_units_precision ();

	wvbox = GTK_WIDGET (size);

	/* ---- W line ---- */
	whbox = gl_hig_hbox_new ();
	gl_hig_vbox_add_widget (GL_HIG_VBOX(wvbox), whbox);

	/* W Label */
	size->w_label = gtk_label_new (_("Width:"));
	gtk_misc_set_alignment (GTK_MISC (size->w_label), 0, 0.5);
	gl_hig_hbox_add_widget (GL_HIG_HBOX(whbox), size->w_label);

	/* W spin */
	w_adjust = gtk_adjustment_new (climb_rate, climb_rate, 100.0,
				       climb_rate, 10.0, 10.0);
	size->w_spin = gtk_spin_button_new (GTK_ADJUSTMENT (w_adjust),
					    climb_rate, digits);
	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (size->w_spin),
					   TRUE);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (size->w_spin), TRUE);
	gl_hig_hbox_add_widget (GL_HIG_HBOX(whbox), size->w_spin);

	/* ---- H line ---- */
	whbox = gl_hig_hbox_new ();
	gl_hig_vbox_add_widget (GL_HIG_VBOX(wvbox), whbox);

	/* H label */
	size->h_label = gtk_label_new (_("Height:"));
	gtk_misc_set_alignment (GTK_MISC (size->h_label), 0, 0.5);
	gl_hig_hbox_add_widget (GL_HIG_HBOX(whbox), size->h_label);

	/* H spin */
	h_adjust = gtk_adjustment_new (climb_rate, climb_rate,
				       100.0, climb_rate, 10.0, 10.0);
	size->h_spin = gtk_spin_button_new (GTK_ADJUSTMENT (h_adjust),
					    climb_rate, digits);
	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (size->h_spin),
					   TRUE);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (size->h_spin), TRUE);
	gl_hig_hbox_add_widget (GL_HIG_HBOX(whbox), size->h_spin);

	/* Units */
	size->units_label = gtk_label_new (units_string);
	gtk_misc_set_alignment (GTK_MISC (size->units_label), 0, 0.5);
	gl_hig_hbox_add_widget (GL_HIG_HBOX(whbox), size->units_label);

	/* Maintain aspect ratio checkbox */
	size->aspect_checkbox =
	    gtk_check_button_new_with_label (_
					     ("Maintain current aspect ratio"));
	gl_hig_vbox_add_widget (GL_HIG_VBOX(wvbox), size->aspect_checkbox);

	/* Connect signals to controls */
	g_signal_connect (G_OBJECT (size->aspect_checkbox), "toggled",
			  G_CALLBACK (aspect_toggle_cb), size);
	g_signal_connect (G_OBJECT (size->w_spin), "changed",
			  G_CALLBACK (w_spin_cb), size);
	g_signal_connect (G_OBJECT (size->h_spin), "changed",
			  G_CALLBACK (h_spin_cb), size);
}

/*--------------------------------------------------------------------------*/
/* PRIVATE.  Maintain aspect ratio checkbox callback.                       */
/*--------------------------------------------------------------------------*/
static void
aspect_toggle_cb (GtkToggleButton *togglebutton,
		  gpointer         user_data)
{
	glWdgtSize *size = GL_WDGT_SIZE (user_data);
	GtkAdjustment *w_adjust, *h_adjust;

	if (gtk_toggle_button_get_active (togglebutton)) {

		size->w =
		    gtk_spin_button_get_value (GTK_SPIN_BUTTON(size->w_spin));
		size->h =
		    gtk_spin_button_get_value (GTK_SPIN_BUTTON(size->h_spin));

		size->aspect_ratio = size->h / size->w;

		/* We have a new aspect ratio, adjust one of the maxes accordingly */
		if ((size->h_max_orig / size->w_max_orig) < size->aspect_ratio) {
			size->w_max = size->h_max_orig / size->aspect_ratio;
			size->h_max = size->h_max_orig;
		} else {
			size->w_max = size->w_max_orig;
			size->h_max = size->w_max_orig * size->aspect_ratio;
		}

	} else {

		/* Reset maximums */
		size->w_max = size->w_max_orig;
		size->h_max = size->h_max_orig;

	}

	g_signal_handlers_block_by_func (GTK_OBJECT (size->w_spin),
					  G_CALLBACK (w_spin_cb),
					  user_data);
	g_signal_handlers_block_by_func (GTK_OBJECT (size->h_spin),
					  G_CALLBACK (h_spin_cb),
					  user_data);
	w_adjust =
	    gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (size->w_spin));
	w_adjust->upper = size->w_max;
	gtk_spin_button_update (GTK_SPIN_BUTTON (size->w_spin));
	h_adjust =
	    gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (size->h_spin));
	h_adjust->upper = size->h_max;
	gtk_spin_button_update (GTK_SPIN_BUTTON (size->h_spin));
	g_signal_handlers_unblock_by_func (GTK_OBJECT (size->w_spin),
					    G_CALLBACK (w_spin_cb),
					    user_data);
	g_signal_handlers_unblock_by_func (GTK_OBJECT (size->h_spin),
					    G_CALLBACK (h_spin_cb),
					    user_data);

	/* Emit our "changed" signal */
	g_signal_emit (G_OBJECT (size), wdgt_size_signals[CHANGED], 0);

}

/*--------------------------------------------------------------------------*/
/* PRIVATE.  W spin button changed callback.                                */
/*--------------------------------------------------------------------------*/
static void
w_spin_cb (GtkSpinButton *spinbutton,
	   gpointer       user_data)
{
	glWdgtSize *size = GL_WDGT_SIZE (user_data);
	GtkToggleButton *toggle = GTK_TOGGLE_BUTTON (size->aspect_checkbox);

	size->w =
	    gtk_spin_button_get_value (GTK_SPIN_BUTTON (size->w_spin));

	if (gtk_toggle_button_get_active (toggle)) {

		size->h = size->w * size->aspect_ratio;

		/* Update our sibling control, blocking recursion. */
		g_signal_handlers_block_by_func (GTK_OBJECT (size->h_spin),
						  G_CALLBACK (h_spin_cb),
						  user_data);
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (size->h_spin),
					   size->h);
		g_signal_handlers_unblock_by_func (GTK_OBJECT (size->h_spin),
						    G_CALLBACK (h_spin_cb),
						    user_data);
	}

	/* Emit our "changed" signal */
	g_signal_emit (G_OBJECT (size), wdgt_size_signals[CHANGED], 0);

}

/*--------------------------------------------------------------------------*/
/* PRIVATE.  H spin button changed callback.                                */
/*--------------------------------------------------------------------------*/
static void
h_spin_cb (GtkSpinButton *spinbutton,
	   gpointer       user_data)
{
	glWdgtSize *size = GL_WDGT_SIZE (user_data);
	GtkToggleButton *toggle = GTK_TOGGLE_BUTTON (size->aspect_checkbox);

	size->h =
	    gtk_spin_button_get_value (GTK_SPIN_BUTTON (size->h_spin));

	if (gtk_toggle_button_get_active (toggle)) {

		size->w = size->h / size->aspect_ratio;

		/* Update our sibling control, blocking recursion. */
		g_signal_handlers_block_by_func (GTK_OBJECT (size->w_spin),
						  G_CALLBACK (w_spin_cb),
						  user_data);
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (size->w_spin),
					   size->w);
		g_signal_handlers_unblock_by_func (GTK_OBJECT (size->w_spin),
						    G_CALLBACK (w_spin_cb),
						    user_data);
	}

	/* Emit our "changed" signal */
	g_signal_emit (G_OBJECT (size), wdgt_size_signals[CHANGED], 0);

}

/***************************************************************************/
/* query values from controls.                                             */
/***************************************************************************/
void
gl_wdgt_size_get_size (glWdgtSize *size,
		       gdouble    *w,
		       gdouble    *h,
		       gboolean   *keep_aspect_ratio_flag)
{
	gdouble units_per_point;

	units_per_point = gl_prefs_get_units_per_point ();

	*w = gtk_spin_button_get_value (GTK_SPIN_BUTTON(size->w_spin));
	*h = gtk_spin_button_get_value (GTK_SPIN_BUTTON(size->h_spin));

	*keep_aspect_ratio_flag =
	    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					  (size->aspect_checkbox));

	/* convert everything back to our internal units (points) */
	*w /= units_per_point;
	*h /= units_per_point;
}

/***************************************************************************/
/* set values and ranges for controls.                                     */
/***************************************************************************/
void
gl_wdgt_size_set_params (glWdgtSize *size,
			 gdouble     w,
			 gdouble     h,
			 gboolean    keep_aspect_ratio_flag,
			 gdouble     w_max,
			 gdouble     h_max)
{
	GtkObject *w_adjust, *h_adjust;
	const gchar *units_string;
	gdouble units_per_point, climb_rate;
	gint digits;

	units_string = gl_prefs_get_units_string ();
	units_per_point = gl_prefs_get_units_per_point ();
	climb_rate = gl_prefs_get_units_step_size ();
	digits = gl_prefs_get_units_precision ();

	/* Put everything into our display units */
	size->w = w * units_per_point;
	size->h = h * units_per_point;
	size->w_max = w_max * units_per_point;
	size->h_max = h_max * units_per_point;

	/* Squirrel away copies of our original maximums */
	size->w_max_orig = size->w_max;
	size->h_max_orig = size->h_max;

	size->aspect_ratio = size->h / size->w;
	if (keep_aspect_ratio_flag) {

		/* When tracking aspect ratio, adjust one of the maxes */
		if ((size->h_max / size->w_max) < size->aspect_ratio) {
			size->w_max = size->h_max / size->aspect_ratio;
		} else {
			size->h_max = size->w_max * size->aspect_ratio;
		}

		/* before adjusting w & h, limit to max values */
		if (size->w > size->w_max)
			size->w = size->w_max;
		if (size->h > size->h_max)
			size->h = size->h_max;

	}

	/* update W/H spin controls */
	w_adjust = gtk_adjustment_new (size->w, climb_rate, size->w_max,
				       climb_rate, 10.0, 10.0);
	gtk_spin_button_configure (GTK_SPIN_BUTTON (size->w_spin),
				   GTK_ADJUSTMENT (w_adjust), climb_rate,
				   digits);
	h_adjust =
	    gtk_adjustment_new (size->h, climb_rate, size->h_max, climb_rate,
				10.0, 10.0);
	gtk_spin_button_configure (GTK_SPIN_BUTTON (size->h_spin),
				   GTK_ADJUSTMENT (h_adjust), climb_rate,
				   digits);

	gtk_label_set_text (GTK_LABEL (size->units_label), units_string);

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (size->aspect_checkbox),
				      keep_aspect_ratio_flag);

}

/***************************************************************************/
/* set size only.                                                          */
/***************************************************************************/
void
gl_wdgt_size_set_size (glWdgtSize *size,
		       gdouble     w,
		       gdouble     h)
{
	gdouble units_per_point;

	units_per_point = gl_prefs_get_units_per_point ();

	/* Put everything into our display units */
	size->w = w * units_per_point;
	size->h = h * units_per_point;

	/* update aspect ratio */
	size->aspect_ratio = size->h / size->w;

	/* update W/H spin controls */
	g_signal_handlers_block_by_func (GTK_OBJECT (size->w_spin),
					 G_CALLBACK (w_spin_cb),
					 size);
	g_signal_handlers_block_by_func (GTK_OBJECT (size->h_spin),
					 G_CALLBACK (h_spin_cb),
					 size);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (size->w_spin), size->w);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (size->h_spin), size->h);
	g_signal_handlers_unblock_by_func (GTK_OBJECT (size->w_spin),
					   G_CALLBACK (w_spin_cb),
					   size);
	g_signal_handlers_unblock_by_func (GTK_OBJECT (size->h_spin),
					   G_CALLBACK (h_spin_cb),
					   size);
}

/****************************************************************************/
/* Set size group for internal labels                                       */
/****************************************************************************/
void
gl_wdgt_size_set_label_size_group (glWdgtSize   *size,
				   GtkSizeGroup *label_size_group)
{
	gtk_size_group_add_widget (label_size_group, size->w_label);
	gtk_size_group_add_widget (label_size_group, size->h_label);
}
